package com.ywt.alipaympapi.service.impl;

import com.ywt.alipaympapi.core.utils.BizUtil;
import com.ywt.alipaympapi.core.utils.Checker;
import com.ywt.alipaympapi.core.utils.ContextHelper;
import com.ywt.alipaympapi.core.utils.StringHelper;
import com.ywt.alipaympapi.models.vo.MedicalCardVo;
import com.ywt.core.exception.AppMessageException;
import com.ywt.gapi.Result;
import com.ywt.gapi.ResultCode;
import com.ywt.gapi.third.taihe.*;
import com.ywt.gapi.user.GetMedicalCardListRequest;
import com.ywt.gapi.user.GetMedicalCardListResponse;
import com.ywt.gapi.user.MedicalCard;
import com.ywt.gapi.user.UserServiceGrpc;
import com.ywt.model.PageVO;
import com.ywt.model.Pagination;
import com.ywt.request.report.QueryReportDetailRequest;
import com.ywt.request.report.QueryReportListRequest;
import com.ywt.response.register.QueryDoctorListResponseData;
import com.ywt.response.report.QueryReportDetailResponseData;
import com.ywt.response.report.QueryReportListResponseData;
import com.ywt.alipaympapi.service.IsvReportService;
import com.ywt.response.report.ReportItem;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.util.ArrayList;
import java.util.List;

@Service
public class IsvReportServiceImpl implements IsvReportService {

    @Autowired
    private UserServiceGrpc.UserServiceBlockingStub userServiceBlockingStub;

    @Autowired
    private TaiheServiceGrpc.TaiheServiceBlockingStub taiheServiceBlockingStub;

    @Override
    public QueryReportDetailResponseData queryReportDetail(QueryReportDetailRequest request) throws AppMessageException {
        QueryReportDetailResponseData data = new QueryReportDetailResponseData();
        if (StringHelper.isNullOrWhiteSpace(request.getRecordId())) {
            throw new AppMessageException("没有检验检查报告Id");
        }

        if (StringHelper.isNullOrWhiteSpace(request.getRecordId())) {
            throw new AppMessageException("没有报告类型");
        }
        try {
            int userId = ContextHelper.getCurrentUserIdWrapped();
            int hospitalId = BizUtil.getCurrentHospitalId();
            String cardNum = request.getCardNum();
            MedicalCardVo card = getCard(userId, hospitalId, cardNum);
            String type = request.getType();
            if ("检查报告".equals(type)) {

                GetInspectionDetailRequest req = GetInspectionDetailRequest.newBuilder()
                        .setReportId(request.getRecordId())
                        .setHospitalId(hospitalId)
                        .build();
                GetInspectionDetailResponse res = taiheServiceBlockingStub.getInspectionDetail(req);
                if (res.getCode() == ResultCode.SUCCEED_VALUE) {
                    List<String> reportItem = new ArrayList<>();
                    InspectionDetail detail = res.getDetail();
                    data.setRecordId(request.getRecordId());
                    data.setUserName(card.getPatientName());
                    data.setUserSex(card.getSex() == 1 ? "男" : "女");
                    data.setCardNum(cardNum);
                    data.setDepartmentName(detail.getDeptName());
                    data.setApplyDoctorName(detail.getDoctorName());
                    reportItem.add(detail.getDiagnose());
                    data.setDiagnosis(reportItem);
                    data.setConfirmDoctorName(detail.getInspectionDoctor());
                    data.setReportDate(detail.getInspectionDate());
                    data.setReportImageDesc(detail.getInspectionResult());

                }
                throw new AppMessageException("没有此报告");
            }
            if ("检验报告".equals(type)) {
                GetLaboratoryDetailRequest req = GetLaboratoryDetailRequest.newBuilder()
                        .setInspectionId(request.getRecordId())
                        .setHospitalId(hospitalId)
                        .build();
                GetLaboratoryDetailResponse res = taiheServiceBlockingStub.getLaboratoryDetail(req);
                if (res.getCode() == ResultCode.SUCCEED_VALUE) {
                    List<ReportItem> reportItemList = new ArrayList<>();
                    List<String> reportItem = new ArrayList<>();

                    data.setItemName(res.getTitle());
                    data.setReportDate(res.getTime());
                    data.setConfirmDoctorName(res.getDoctor());
                    for (LaboratoryDetail detail : res.getDetailList()) {
                        ReportItem r = new ReportItem();
                        r.setItemName(detail.getItemName());
                        String flag = detail.getResultFlag() == 1 ? "HIGH" : detail.getResultFlag() == 2 ? "LOW" : "NORMAL";
                        r.setPnFlag(flag);
                        r.setReference(detail.getItemRef());
                        r.setTxtVal(detail.getTestResult());
                        r.setUnit(detail.getUnit());
                        reportItemList.add(r);
                    }
                    data.setReportItemList(reportItemList);

                }
                throw new AppMessageException("没有此报告");
            }
        } catch (Exception e) {
            e.printStackTrace();
        }
        return data;
    }

    @Override
    public PageVO<QueryReportListResponseData> queryReportList(QueryReportListRequest request) {
         List<QueryReportListResponseData> list = new ArrayList<>();
        int userId = ContextHelper.getCurrentUserIdWrapped();
        String cardNum = request.getCardNum();
        // 报告类型：分为"检查报告"、"检验报告"两种
        String type = request.getType();
        int hospitalId = BizUtil.getCurrentHospitalId();
        int pageIndex = request.getPageIndex();
        int pageSize = request.getPageSize();
        String hisPatientId = request.getHisPatientId();
        int total = 0;
        try {
            MedicalCardVo card = getCard(userId, hospitalId, hisPatientId);

            if ("检查报告".equals(type)) {
                GetInspectionListRequest getInspectionListRequest = GetInspectionListRequest.newBuilder()
                        .setPatientId(hisPatientId)
                        .setHospitalId(hospitalId)
                        .build();
                GetInspectionListResponse res = taiheServiceBlockingStub.getInspectionList(getInspectionListRequest);
                if (res.getCode() == ResultCode.SUCCEED_VALUE) {
                    total = res.getInspectionReportList().size();
                    for (InspectionReport q : res.getInspectionReportList()) {
                        QueryReportListResponseData da = new QueryReportListResponseData();
                        da.setCardNum(cardNum);
                        da.setItemName(Checker.getStringValue(q.getReportTitle()));
                        da.setRecordId(Checker.getStringValue(q.getReportId()));
                        list.add(da);
                    }
                }

            }
            if ("检验报告".equals(type)) {
                GetLaboratoryListRequest req = GetLaboratoryListRequest.newBuilder()
                        .setPatientId(hisPatientId)
                        .setHospitalId(hospitalId)
                        .build();
                GetLaboratoryListResponse thRes = taiheServiceBlockingStub.getLaboratoryList(req);
                if (thRes.getCode() == ResultCode.SUCCEED_VALUE) {
                    total = thRes.getReportList().size();
                    for (LaboratoryReport q : thRes.getReportList()) {
                        QueryReportListResponseData da = new QueryReportListResponseData();
                        da.setCardNum(cardNum);
                        da.setItemName(Checker.getStringValue(q.getReportTitle()));
                        da.setDepartmentName(Checker.getStringValue(q.getDeptName()));
                        da.setRecordId(Checker.getStringValue(q.getReportId()));
                        list.add(da);
                    }
                }
            }
        } catch (Exception e) {
            e.printStackTrace();
        }
        Pagination pagination = new Pagination(pageIndex, pageSize, total);
        PageVO<QueryReportListResponseData> res = new PageVO<>(pagination, list);
        return res;
    }


    /**
     * 根据 userId & hospitalId 获取用户可用就诊卡列表
     */
    public List<MedicalCard> getUserMedCardList(int userId, int hospitalId) throws AppMessageException {
        GetMedicalCardListRequest getMedicalCardListRequest = GetMedicalCardListRequest.newBuilder()
                .setHospitalId(hospitalId)
                .setUserId(userId)
                .build();
        GetMedicalCardListResponse getMedicalCardListResponse = userServiceBlockingStub.getMedicalCardList(getMedicalCardListRequest);
        Result result = getMedicalCardListResponse.getResult();
        if (result.getCode() != ResultCode.SUCCEED) throw new AppMessageException(result.getInfo());
        return getMedicalCardListResponse.getMedicalCardsList();
    }

    public MedicalCardVo getCard(int userId, int hospitalId, String hisPatientId) throws Exception {
        List<MedicalCard> list = getUserMedCardList(userId, hospitalId);
        if (Checker.isNone(list)) throw new AppMessageException("没有诊疗卡");
        for(MedicalCard card: list){
            if(hisPatientId.equals(card.getPatientId())){
                MedicalCardVo v = new MedicalCardVo();
                v.setSex(card.getSex());
                v.setHisPatientId(card.getPatientId());
                v.setPatientName(card.getPatientName());
                return v;
            }
        }

        throw new AppMessageException("没有此报告");
    }
}
